@@ -91,7 +91,7 @@ import botocore.exceptions
 from botocore.vendored.requests import ConnectionError as \
     RequestsConnectionError
 from botocore.session import Session  # noqa
-from typing import Optional, Dict, List, Any, Type  # noqa
+from typing import Optional, Dict, List, Any, Type, cast  # noqa
 
 from chalice.config import Config  # noqa
 from chalice.compat import is_broken_pipe_error
@@ -105,6 +105,8 @@ from chalice.constants import DEFAULT_LAMBDA_TIMEOUT
 from chalice.constants import DEFAULT_LAMBDA_MEMORY_SIZE
 from chalice.constants import DEFAULT_TLS_VERSION
 from chalice.constants import SQS_EVENT_SOURCE_POLICY
+from chalice.constants import KINESIS_EVENT_SOURCE_POLICY
+from chalice.constants import DDB_EVENT_SOURCE_POLICY
 from chalice.constants import POST_TO_WEBSOCKET_CONNECTION_POLICY
 from chalice.deploy import models
 from chalice.deploy.appgraph import ApplicationGraphBuilder, DependencyBuilder
@@ -115,6 +117,10 @@ from chalice.deploy.packager import PipRunner
 from chalice.deploy.packager import SubprocessPip
 from chalice.deploy.packager import DependencyBuilder as PipDependencyBuilder
 from chalice.deploy.packager import LambdaDeploymentPackager
+from chalice.deploy.packager import AppOnlyDeploymentPackager
+from chalice.deploy.packager import LayerDeploymentPackager
+from chalice.deploy.packager import BaseLambdaDeploymentPackager  # noqa
+from chalice.deploy.packager import EmptyPackageError
 from chalice.deploy.planner import PlanStage
 from chalice.deploy.planner import RemoteState
 from chalice.deploy.planner import NoopPlanner
@@ -268,7 +274,7 @@ def _create_deployer(session,       # type: Session
         application_builder=ApplicationGraphBuilder(),
         deps_builder=DependencyBuilder(),
         build_stage=create_build_stage(
-            osutils, UI(), TemplatedSwaggerGenerator(),
+            osutils, UI(), TemplatedSwaggerGenerator(), config
         ),
         plan_stage=PlanStage(
             osutils=osutils, remote_state=RemoteState(
@@ -280,24 +286,40 @@ def _create_deployer(session,       # type: Session
     )
 
 
-def create_build_stage(osutils, ui, swagger_gen):
-    # type: (OSUtils, UI, SwaggerGenerator) -> BuildStage
+def create_build_stage(osutils, ui, swagger_gen, config):
+    # type: (OSUtils, UI, SwaggerGenerator, Config) -> BuildStage
     pip_runner = PipRunner(pip=SubprocessPip(osutils=osutils),
                            osutils=osutils)
     dependency_builder = PipDependencyBuilder(
         osutils=osutils,
         pip_runner=pip_runner
     )
+    deployment_packager = cast(BaseDeployStep, None)
+    if config.automatic_layer:
+        deployment_packager = ManagedLayerDeploymentPackager(
+            lambda_packager=AppOnlyDeploymentPackager(
+                osutils=osutils,
+                dependency_builder=dependency_builder,
+                ui=ui,
+            ),
+            layer_packager=LayerDeploymentPackager(
+                osutils=osutils,
+                dependency_builder=dependency_builder,
+                ui=ui,
+            )
+        )
+    else:
+        deployment_packager = DeploymentPackager(
+            packager=LambdaDeploymentPackager(
+                osutils=osutils,
+                dependency_builder=dependency_builder,
+                ui=ui,
+            )
+        )
     build_stage = BuildStage(
         steps=[
             InjectDefaults(),
-            DeploymentPackager(
-                packager=LambdaDeploymentPackager(
-                    osutils=osutils,
-                    dependency_builder=dependency_builder,
-                    ui=ui,
-                ),
-            ),
+            deployment_packager,
             PolicyGenerator(
                 policy_gen=AppPolicyGenerator(
                     osutils=osutils
@@ -362,6 +384,9 @@ class Deployer(object):
             config, chalice_stage_name)
         resources = self._deps_builder.build_dependencies(application)
         self._build_stage.execute(config, resources)
+        # Rebuild dependencies in case the build stage modified
+        # the app graph.
+        resources = self._deps_builder.build_dependencies(application)
         plan = self._plan_stage.execute(resources)
         self._sweeper.execute(plan, config)
         self._executor.execute(plan)
@@ -430,6 +455,51 @@ class DeploymentPackager(BaseDeployStep):
             resource.filename = zip_filename
 
 
+class ManagedLayerDeploymentPackager(BaseDeployStep):
+    # If we're creating a layer for non-app code there's two different
+    # packagers we need.  One for the Lambda functions (app code) and
+    # one for the Lambda layer (requirements.txt + vendor).
+    def __init__(self,
+                 lambda_packager,  # type: BaseLambdaDeploymentPackager
+                 layer_packager,   # type: BaseLambdaDeploymentPackager
+                 ):
+        # type: (...) -> None
+        self._lambda_packager = lambda_packager
+        self._layer_packager = layer_packager
+
+    def handle_lambdafunction(self, config, resource):
+        # type: (Config, models.LambdaFunction) -> None
+        if isinstance(resource.deployment_package.filename,
+                      models.Placeholder):
+            zip_filename = self._lambda_packager.create_deployment_package(
+                config.project_dir, config.lambda_python_version
+            )
+            resource.deployment_package.filename = zip_filename
+        if resource.managed_layer is not None and \
+                resource.managed_layer.is_empty:
+            # Lambda doesn't allow us to create an empty layer so if we've
+            # tried to create the deploment package and determined it's empty
+            # we should remove the managed layer from the model entirely so
+            # downstream consumers don't have to worry about it.
+            resource.managed_layer = None
+
+    def handle_lambdalayer(self, config, resource):
+        # type: (Config, models.LambdaLayer) -> None
+        if isinstance(resource.deployment_package.filename,
+                      models.Placeholder):
+            try:
+                zip_filename = self._layer_packager.create_deployment_package(
+                    config.project_dir, config.lambda_python_version
+                )
+                resource.deployment_package.filename = zip_filename
+            except EmptyPackageError:
+                # An empty package is not valid in Lambda.  There's
+                # no point in trying to represent it in the model
+                # because nothing downstream (planner, SAM/tf) can
+                # consume as a useful entity.
+                resource.is_empty = True
+
+
 class SwaggerBuilder(BaseDeployStep):
     def __init__(self, swagger_generator):
         # type: (SwaggerGenerator) -> None
@@ -445,7 +515,9 @@ class SwaggerBuilder(BaseDeployStep):
 class LambdaEventSourcePolicyInjector(BaseDeployStep):
     def __init__(self):
         # type: () -> None
-        self._policy_injected = False
+        self._sqs_policy_injected = False
+        self._kinesis_policy_injected = False
+        self._ddb_policy_injected = False
 
     def handle_sqseventsource(self, config, resource):
         # type: (Config, models.SQSEventSource) -> None
@@ -453,14 +525,48 @@ class LambdaEventSourcePolicyInjector(BaseDeployStep):
         # available records so the lambda function needs
         # permission to call sqs.
         role = resource.lambda_function.role
-        if (not self._policy_injected and
+        if not self._sqs_policy_injected and \
+                self._needs_policy_injected(role):
+            # mypy can't follow the type narrowing from
+            # _needs_policy_injected so we're working around
+            # that by explicitly casting the role.
+            role = cast(models.ManagedIAMRole, role)
+            document = cast(Dict[str, Any], role.policy.document)
+            self._inject_trigger_policy(document,
+                                        SQS_EVENT_SOURCE_POLICY.copy())
+            self._sqs_policy_injected = True
+
+    def handle_kinesiseventsource(self, config, resource):
+        # type: (Config, models.KinesisEventSource) -> None
+        role = resource.lambda_function.role
+        if not self._kinesis_policy_injected and \
+                self._needs_policy_injected(role):
+            # See commen in handle_kinesiseventsource about this cast.
+            role = cast(models.ManagedIAMRole, role)
+            document = cast(Dict[str, Any], role.policy.document)
+            self._inject_trigger_policy(document,
+                                        KINESIS_EVENT_SOURCE_POLICY.copy())
+            self._kinesis_policy_injected = True
+
+    def handle_dynamodbeventsource(self, config, resource):
+        # type: (Config, models.KinesisEventSource) -> None
+        role = resource.lambda_function.role
+        if not self._ddb_policy_injected and \
+                self._needs_policy_injected(role):
+            # See commen in handle_kinesiseventsource about this cast.
+            role = cast(models.ManagedIAMRole, role)
+            document = cast(Dict[str, Any], role.policy.document)
+            self._inject_trigger_policy(document,
+                                        DDB_EVENT_SOURCE_POLICY.copy())
+            self._ddb_policy_injected = True
+
+    def _needs_policy_injected(self, role):
+        # type: (models.IAMRole) -> bool
+        return (
             isinstance(role, models.ManagedIAMRole) and
             isinstance(role.policy, models.AutoGenIAMPolicy) and
-            not isinstance(role.policy.document,
-                           models.Placeholder)):
-            self._inject_trigger_policy(role.policy.document,
-                                        SQS_EVENT_SOURCE_POLICY.copy())
-            self._policy_injected = True
+            not isinstance(role.policy.document, models.Placeholder)
+        )
 
     def _inject_trigger_policy(self, document, policy):
         # type: (Dict[str, Any], Dict[str, Any]) -> None
@@ -625,6 +731,11 @@ class DeploymentReporter(object):
         # type: (Dict[str, Any], List[str]) -> None
         report.append('  - Lambda ARN: %s' % resource['lambda_arn'])
 
+    def _report_lambda_layer(self, resource, report):
+        # type: (Dict[str, Any], List[str]) -> None
+        report.append('  - Lambda Layer ARN: %s' % (
+            resource['layer_version_arn']))
+
     def _default_report(self, resource, report):
         # type: (Dict[str, Any], List[str]) -> None
         # The default behavior is to not report a resource.  This
