@@ -1,3 +1,4 @@
+# pylint: disable=too-many-lines
 import sys
 import hashlib
 import inspect
@@ -59,6 +60,14 @@ class EmptyPackageError(Exception):
     """A deployment package cannot be an empty zip file."""
 
 
+class UnsupportedPackageError(Exception):
+    """Unable to parse package metadata."""
+    def __init__(self, package_name):
+        # type: (str) -> None
+        super(UnsupportedPackageError, self).__init__(
+            'Unable to retrieve name/version for package: %s' % package_name)
+
+
 class BaseLambdaDeploymentPackager(object):
     _CHALICE_LIB_DIR = 'chalicelib'
     _VENDOR_DIR = 'vendor'
@@ -428,7 +437,9 @@ class DependencyBuilder(object):
     packager.
     """
     _MANYLINUX_COMPATIBLE_PLATFORM = {'any', 'linux_x86_64',
-                                      'manylinux1_x86_64'}
+                                      'manylinux1_x86_64',
+                                      'manylinux2010_x86_64',
+                                      'manylinux2014_x86_64'}
     _COMPATIBLE_PACKAGE_WHITELIST = {
         'sqlalchemy',
         'pyyaml',
@@ -816,7 +827,7 @@ class SDistMetadataFetcher(object):
         parser.feed(data)
         return parser.close()
 
-    def _generate_egg_info(self, package_dir):
+    def _get_pkg_info_filepath(self, package_dir):
         # type: (str) -> str
         setup_py = self._osutils.joinpath(package_dir, 'setup.py')
         script = self._SETUPTOOLS_SHIM % setup_py
@@ -827,10 +838,23 @@ class SDistMetadataFetcher(object):
         self._osutils.makedirs(egg_info_dir)
         p = subprocess.Popen(cmd, cwd=package_dir,
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        p.communicate()
+        _, stderr = p.communicate()
+        if p.returncode != 0:
+            logger.debug("Non zero rc (%s) from the setup.py egg_info "
+                         "command: %s", p.returncode, stderr)
         info_contents = self._osutils.get_directory_contents(egg_info_dir)
-        pkg_info_path = self._osutils.joinpath(
-            egg_info_dir, info_contents[0], 'PKG-INFO')
+        if info_contents:
+            pkg_info_path = self._osutils.joinpath(
+                egg_info_dir, info_contents[0], 'PKG-INFO')
+        else:
+            # This might be a pep 517 package in which case this PKG-INFO file
+            # should be available right in the top level directory of the sdist
+            # in the case where the egg_info command fails.
+            logger.debug("Using fallback location for PKG-INFO file in "
+                         "package directory: %s", package_dir)
+            pkg_info_path = self._osutils.joinpath(package_dir, 'PKG-INFO')
+        if not self._osutils.file_exists(pkg_info_path):
+            raise UnsupportedPackageError(self._osutils.basename(package_dir))
         return pkg_info_path
 
     def _unpack_sdist_into_dir(self, sdist_path, unpack_dir):
@@ -849,7 +873,7 @@ class SDistMetadataFetcher(object):
         # type: (str) -> Tuple[str, str]
         with self._osutils.tempdir() as tempdir:
             package_dir = self._unpack_sdist_into_dir(sdist_path, tempdir)
-            pkg_info_filepath = self._generate_egg_info(package_dir)
+            pkg_info_filepath = self._get_pkg_info_filepath(package_dir)
             metadata = self._parse_pkg_info_file(pkg_info_filepath)
             name = metadata['Name']
             version = metadata['Version']
@@ -950,8 +974,8 @@ class PipRunner(object):
             if err is None:
                 err = b'Unknown error'
             error = err.decode()
-            match = re.search(("Could not find a version that satisfies the "
-                               "requirement (.+?) "), error)
+            match = re.search(r"Could not find a version that satisfies the "
+                              r"requirement ([^\s]+)", error)
             if match:
                 package_name = match.group(1)
                 raise NoSuchPackageError(str(package_name))
@@ -983,7 +1007,7 @@ class PipRunner(object):
         # version and is checked later.
         for package in packages:
             arguments = ['--only-binary=:all:', '--no-deps', '--platform',
-                         'manylinux1_x86_64', '--implementation', 'cp',
+                         'manylinux2014_x86_64', '--implementation', 'cp',
                          '--abi', abi, '--dest', directory, package]
             self._execute('download', arguments)
 
